#ifndef CAPTION_HPP
#define CAPTION_HPP

#include <string>

#include <basic_widgets/abstract/font_handler.hpp>
#include <basic_widgets/abstract/widget.hpp>

struct SDL_Surface;
struct SDL_Texture;

namespace bwidgets::widget
{
    class Caption : public abstract::Widget, public abstract::FontHandler
    {
        public:
            enum struct AlignmentH {
                CENTER,
                LEFT,
                RIGHT
            };

            enum struct AlignmentV {
                BOTTOM,
                CENTER,
                TOP
            };

        private:
            SDL_Color     _color_fg     {0, 0, 0, SDL_ALPHA_OPAQUE};
            std::string   _text;
            SDL_Surface*  _text_surface {nullptr};
            SDL_Texture*  _text_texture {nullptr};
            SDL_Renderer* _renderer     {nullptr};

            void _handle_font_change();
            void _render_text();

        public:
            struct {
                AlignmentH h;
                AlignmentV v;
            }                      alignment {AlignmentH::LEFT, AlignmentV::CENTER};
            SDL_Color              color_bg  {0, 0, 0, SDL_ALPHA_TRANSPARENT};
            abstract::Widget::Size margins   {3, 0};

            Caption(Widget* parent=nullptr);
            ~Caption();

            void                   color_fg(const SDL_Color&);
            void                   render(SDL_Renderer*);
            abstract::Widget::Size size()                     const;
            const std::string&     text()                     const;
            void                   text(const std::string&);
    };
}

#endif
