#include <SDL2/SDL_render.h>

#include <basic_widgets/button.hpp>
#include <basic_widgets/utils/math.hpp>
#include <iostream>

using namespace bwidgets;
using namespace bwidgets::utils;

widget::Button::Button(Widget* parent)
    : abstract::Widget(parent),
    _caption(parent)
{
    _focus_area = _click_area = &_widget_area;
    _caption.alignment.h = Caption::AlignmentH::CENTER;
}

void widget::Button::render(SDL_Renderer* r)
{
    Widget::render(r);

    SDL_RenderSetViewport(r, &_viewport);

    // TODO: smoothstep it
    auto color_shade = [](int base, int step, int total_steps) -> int {
        int steps_to_max = 255 - base;
        return base + step * steps_to_max / total_steps;
    };

    SDL_Color bg_color = _is_hovered ? color_bg_hover : color_bg;
    if (_is_pushed)
    {
        for (int i = 0; i < border_width; i++)
        {
            SDL_SetRenderDrawColor(
                    r,
                    color_shade(bg_color.r, i, border_width),
                    color_shade(bg_color.g, i, border_width),
                    color_shade(bg_color.b, i, border_width),
                    bg_color.a
                    );
            SDL_Rect rect = {
                _widget_area.x + i,
                _widget_area.y + i,
                _widget_area.w - i * 2,
                _widget_area.h - i * 2
            };
            SDL_RenderFillRect(r, &rect);
        }
    }
    else
    {
        for (int i = border_width; i >= 0; i--)
        {
            SDL_SetRenderDrawColor(
                    r,
                    color_shade(bg_color.r, i, border_width),
                    color_shade(bg_color.g, i, border_width),
                    color_shade(bg_color.b, i, border_width),
                    bg_color.a
                    );
            int margin = border_width - i;
            SDL_Rect rect = {
                _widget_area.x + margin,
                _widget_area.y + margin,
                _widget_area.w - margin * 2,
                _widget_area.h - margin * 2
            };
            SDL_RenderFillRect(r, &rect);
        }
    }

    _caption.render(r);
}

abstract::Widget::Size widget::Button::size() const
{
    return {
        (int)(_caption.size().w * _caption_height_multiplicator)
            + 2 * border_width,
        (int)(_caption.size().h * _caption_height_multiplicator)
            + 2 * border_width
    };
}

const std::string& widget::Button::text() const
{
    return _caption.text();
}

void widget::Button::text(std::string txt)
{
    _caption.text(txt);
    _update_widget_area();
}

void widget::Button::_handle_font_change()
{
    _caption.font(_font);
    _update_widget_area();
}

void widget::Button::_update_widget_area()
{
    int h = _caption.size().h * _caption_height_multiplicator
        + 2 * border_width;

    _widget_area = {
        0,
        math::center_rect(
            _viewport.h,
            _caption.size().h * _caption_height_multiplicator
            ) - border_width,
        _viewport.w,
        h
    };

    auto txt_size = _caption.size();

    _caption_area = {
        math::center_rect(_viewport.w, txt_size.w),
        math::center_rect(_viewport.h, txt_size.h),
        txt_size.w,
        txt_size.h
    };
    _caption.viewport({
            _viewport.x + _caption_area.x,
            _viewport.y + _caption_area.y,
            _caption_area.w,
            _caption_area.h
            });
}
