#include <SDL2/SDL_render.h>
#include <SDL2/SDL_ttf.h>

#include <basic_widgets/caption.hpp>
#include <basic_widgets/utils/math.hpp>
#include <iostream>

using namespace bwidgets;
using namespace bwidgets::utils;

widget::Caption::Caption(abstract::Widget* parent)
    : abstract::Widget(parent)
{
}

widget::Caption::~Caption()
{
    SDL_FreeSurface(_text_surface);
    SDL_DestroyTexture(_text_texture);
}

void widget::Caption::color_fg(const SDL_Color& c)
{
    _color_fg = c;
    _render_text();
}

void widget::Caption::render(SDL_Renderer* r)
{
    Widget::render(r);

    SDL_RenderSetViewport(r, &_viewport);

    SDL_SetRenderDrawColor(r, color_bg.r, color_bg.g, color_bg.b, color_bg.a);
    SDL_RenderFillRect(r, &_widget_area);

    if (_text_surface == nullptr)
        return;

    if (_text_texture == nullptr || _renderer != r)
    {
        _renderer = r;
        _text_texture = SDL_CreateTextureFromSurface(r, _text_surface);
    }

    abstract::Widget::Size size_dst {
        (int)((float)_text_surface->w / _text_surface->h * _viewport.h),
        _viewport.h
    };
    SDL_Rect texture_dst {
        margins.w,
        utils::math::center_rect(_viewport.h, size_dst.h),
        size_dst.w,
        size_dst.h
    };
    switch (alignment.h)
    {
        case AlignmentH::CENTER:
            texture_dst.x = utils::math::center_rect(_viewport.w, texture_dst.w);
            break;
        case AlignmentH::LEFT:
            break;
        case AlignmentH::RIGHT:
            texture_dst.x = _viewport.w - texture_dst.w - margins.w;
            break;
    }
    switch (alignment.v)
    {
        case AlignmentV::BOTTOM:
            texture_dst.y = _viewport.h - texture_dst.h - margins.h;
            break;
        case AlignmentV::CENTER:
            break;
        case AlignmentV::TOP:
            texture_dst.y = margins.h;
    }
    SDL_RenderCopy(
            r,
            _text_texture,
            NULL,
            &texture_dst
            );
}

abstract::Widget::Size widget::Caption::size() const
{
    if (_font == nullptr)
        return {-1, -1};

    abstract::Widget::Size size;
    TTF_SizeUTF8(_font, _text.c_str(), &size.w, &size.h);

    return size;
}

const std::string& widget::Caption::text() const
{
    return _text;
}

void widget::Caption::text(const std::string& t)
{
    _text = t;
    _render_text();
}

void widget::Caption::_handle_font_change()
{
    _render_text();
}

void widget::Caption::_render_text()
{
    if (_font == nullptr)
        return;

    SDL_FreeSurface(_text_surface);
    _text_surface = TTF_RenderUTF8_Blended(
            _font,
            _text.c_str(),
            _color_fg
            );
    SDL_DestroyTexture(_text_texture);
    _text_texture = nullptr;

    _update_widget_area();
}
