#include <SDL2/SDL_rect.h>
#include <SDL2/SDL_events.h>

#include <basic_widgets/abstract/mouse_handler.hpp>
#include <basic_widgets/utils/math.hpp>

using namespace bwidgets;
using namespace bwidgets::utils;

void abstract::MouseHandler::handle_mouse(const SDL_MouseButtonEvent& ev,
                                          const SDL_Rect& orig)
{
    if (_click_area == nullptr)
        return;

    SDL_Point p {ev.x, ev.y};
    SDL_Rect vp {math::rect_offset(*_click_area, orig)};

    if (ev.type == SDL_MOUSEBUTTONDOWN)
    {
        if (SDL_PointInRect(&p, &vp))
        {
            _is_pushed = true;
            _handle_mouse_button(ev, vp);
        }
        else
            focus(false);
    }
    else
    {
        if (_is_pushed)
        {
            if (SDL_PointInRect(&p, &vp))
            {
                focus(true);

                if (click_handler != nullptr)
                    click_handler(ev);
            }
            _is_pushed = false;
            _handle_mouse_button(ev, vp);
        }
    }
}

void abstract::MouseHandler::handle_mouse(const SDL_MouseMotionEvent& ev,
                                          const SDL_Rect& orig)
{
    if (_click_area == nullptr)
        return;

    SDL_Point p {ev.x, ev.y};
    SDL_Rect vp {math::rect_offset(*_click_area, orig)};

    if (SDL_PointInRect(&p, &vp))
        _is_hovered = true;
    else
        _is_hovered = false;

    if (_has_focus)
        _handle_mouse_motion(ev, orig);
}

void abstract::MouseHandler::_handle_mouse_button(const SDL_MouseButtonEvent&,
                                                  const SDL_Rect&)
{
}

void abstract::MouseHandler::_handle_mouse_motion(const SDL_MouseMotionEvent&,
                                                  const SDL_Rect&)
{
}
