#include <vector>

#include <basic_widgets/utils/math.hpp>
#include <basic_widgets/utils/render.hpp>

namespace bwidgets::utils::render
{
    SDL_Color aa(
            const SDL_Color& base_color,
            const int        aa_pixels,
            const float      d
            )
    {
        const auto d_clamp {std::abs(std::clamp<float>(d, -aa_pixels, 0))};
        const auto d_norm  {d_clamp / aa_pixels};
        const auto factor  {3 * d_norm * d_norm - 2 * d_norm * d_norm * d_norm};

        return {
            base_color.r, base_color.g, base_color.b,
                static_cast<Uint8>(base_color.a * factor)
        };
    }

    SDL_Texture* filled_circle(
            const SDL_Color& c,
            const int        resolution,
            SDL_Renderer*    r,
            const int        aa_pixels
            )
    {
        auto            texture {SDL_CreateTexture(
                r,
                SDL_PIXELFORMAT_RGBA32,
                SDL_TEXTUREACCESS_STATIC,
                resolution, resolution)};
        const auto      radius  {resolution / 2};
        const SDL_Point center  {radius, radius};

        bwidgets::utils::render::set_pixels_color(
                texture,
                [aa_pixels, c, center, radius]
                (const SDL_Point& p, const SDL_PixelFormat* format) -> Uint32 {
                const auto d_delta  = math::distance(center, p) - radius;
                const auto aa_color = aa(c, aa_pixels, d_delta);

                return SDL_MapRGBA(format,
                        aa_color.r, aa_color.g, aa_color.b,
                        aa_color.a);
                }
                );

        SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_BLEND);
        SDL_SetTextureScaleMode(texture, SDL_ScaleModeNearest);

        return texture;
    }

    void set_pixels_color(
            SDL_Texture* t,
            std::function<Uint32 (const SDL_Point&, const SDL_PixelFormat*)> pixel_color
            )
    {
        int w, h;
        Uint32 format_enum;
        SDL_QueryTexture(t, &format_enum, NULL, &w, &h);

        auto format = SDL_AllocFormat(format_enum);
        auto pitch = w * format->BytesPerPixel;

        std::vector<Uint32> pixels;
        pixels.reserve(h * pitch);
        for (auto y = 0; y < h; y++)
        {
            for (auto x = 0; x < w; x++)
            {
                pixels.push_back(pixel_color({x, y}, format));
            }
        }

        SDL_UpdateTexture(t, NULL, (const void*)pixels.data(), pitch);

        SDL_FreeFormat(format);
    }
}
