#include <SDL2/SDL_events.h>
#include <SDL2/SDL_render.h>

#include <basic_widgets/abstract/keyboard_handler.hpp>
#include <basic_widgets/abstract/mouse_handler.hpp>
#include <basic_widgets/abstract/widget.hpp>

using namespace bwidgets;

abstract::Widget::Widget(Widget* p)
    : parent(p)
{
}

abstract::Widget::~Widget()
{
}

void abstract::Widget::handle_event(const SDL_Event &ev)
{
    if (auto handler = dynamic_cast<KeyboardHandler*>(this);
             handler != nullptr)
    {
        switch (ev.type)
        {
            case SDL_KEYDOWN:
            case SDL_KEYUP:
                handler->handle_keyboard(ev.key);
                break;
            case SDL_TEXTINPUT:
                handler->handle_keyboard(ev.text);
                break;
        }
    }

    if (auto handler = dynamic_cast<MouseHandler*>(this);
             handler != nullptr)
    {
        switch (ev.type)
        {
            case SDL_MOUSEBUTTONDOWN:
            case SDL_MOUSEBUTTONUP:
                handler->handle_mouse(ev.button, _viewport);
                break;
            case SDL_MOUSEMOTION:
                handler->handle_mouse(ev.motion, _viewport);
                break;
        }
    }
}

void abstract::Widget::render(SDL_Renderer* r)
{
    SDL_RenderSetViewport(r, NULL);
    SDL_Rect dbg_outline = {
        _viewport.x - 1, _viewport.y - 1,
        _viewport.w + 2, _viewport.h + 2
    };
    SDL_SetRenderDrawColor(r, 0, 255, 0, SDL_ALPHA_TRANSPARENT);
    SDL_RenderDrawRect(r, &dbg_outline);
}

void abstract::Widget::viewport(const SDL_Rect& vp)
{
    _viewport = vp;
    _update_widget_area();
}

void abstract::Widget::_update_widget_area()
{
    _widget_area = {
        0,
        0,
        _viewport.w,
        _viewport.h
    };
}

